"""
Yann Thorimbert 2019
"""
from __future__ import print_function, division
import math
import numpy as np
import random
import stlwriter

input_file = "./Points_bathy_xyz_MN95.dbf"

with open(input_file, "r", encoding="latin-1") as f:
    lines = f.readlines()

#First we put the data in a suitable format
lines = lines[2:] #the first 2 lines are not useful here
hmap = []
assert len(lines) == 1
line = [v for v in lines[0].replace("\n"," ").replace("\x1a"," ").split(" ") if v]

x,y,z = [], [], []
for i,v in enumerate(line):
    m = i%3
    if m == 0:
        x.append(float(v))
    elif m == 1:
        y.append(float(v))
    elif m == 2:
        z.append(float(v))

print("L before =", len(x))


#Now we crop the domain in order to include only a given portion
# (you may need do modify this part which is not very general)
x_cut_factor = 0.1
y_cut_factor = 0.25
cut_domain_x = min(x) + x_cut_factor * (max(x)-min(x))
cut_domain_y = min(y) + y_cut_factor * (max(y)-min(y))
to_delete = set()
for i,value in enumerate(y):
    if value > cut_domain_y:
        to_delete.add(i)
for i,value in enumerate(x):
    if value < cut_domain_x:
        to_delete.add(i)
to_delete = list(to_delete)
to_delete.sort(reverse=True)
for i in to_delete:
    x.pop(i)
    y.pop(i)
    z.pop(i)

print("L after =", len(x))


#Now we supress random points in order to reduce the number of vortices...
N = 5000 #arbitrary ! This is the number of points you want to keep
# N = float("inf")
L = len(x)
while len(x) > N:
    index = random.randint(0, len(x)-1)
    x.pop(index)
    y.pop(index)
    z.pop(index)

N = len(x)
print("L final =", len(x))
assert len(x) == len(y) == len(z)
mx, Mx = min(x), max(x)
my, My = min(y), max(y)
mz, Mz = min(z), max(z)
lx = Mx - mx
ly = My - my
lz = Mz - mz

# Now we do the following: we take a regular 2D grid and for each node, we look
# at the nearest point i from the cloudy representation stored in x and y. This
# node's z-value is then z[i]. For nodes further from any point than a given
# threshold D, the value is set to 0.

delta = 2. #arbitrary (look at the units of the raw data file)

# def get_nearest_height(xcoord, ycoord):
#     best_distance = float("inf")
#     best_i = None
#     for i in range(N):
#         distance = math.hypot(x[i]-xcoord, y[i]-ycoord)
#         if distance < best_distance:
#             best_distance = distance
#             best_i = i
#             # if distance < d:
#             #     break
#     if best_distance > D:
#         return Mz
#     return z[best_i]

def get_nearest_height(xcoord, ycoord):
    neigh = []
    for i in range(N):
        distance = math.hypot(x[i]-xcoord, y[i]-ycoord)
        neigh.append((distance,i))
    neigh.sort()
    height =  z[neigh[0][1]] + z[neigh[1][1]] + z[neigh[2][1]] + z[neigh[3][1]]
    return height/4.


nx = int((Mx-mx)/delta)
ny = int((My-my)/delta)
print("Building hmap")
print(N, lx,ly, nx,ny)
hmap = np.zeros((nx,ny), dtype=float)
for ix in range(nx):
    print(round(ix*100/nx), "%")
    xcoord = mx + ix*delta
    for iy in range(ny):
        ycoord = my + iy*delta
        hmap[ix,iy] = get_nearest_height(xcoord, ycoord)


stlwriter.write_stl(np.flipud(hmap.T), "output_with_no_roof.stl", dh=1., dx=1.)

#optional :
import matplotlib.pyplot as plt
plt.imshow(hmap, interpolation="none")
plt.show()

#before erosion
roof_height = mz + 1.1*lz
stlwriter.write_stl_with_a_roof(np.flipud(hmap.T), roof_height, "inner_bed.stl", dh=1., dx=1.)

roof_height = mz-0.5*lz
stlwriter.write_stl_with_a_roof(np.flipud(hmap.T), roof_height, "visu.stl", dh=1., dx=1.)

#after erosion
roof_height = mz + 1.1*lz
hmap[:,:] = mz
stlwriter.write_stl_with_a_roof(np.flipud(hmap.T), roof_height, "outer_bed.stl", dh=1., dx=1.)
#now you can visualize the stl with any program
