"""Takes an hmap as input, gives ASCII stl as output
#-------------------------------------------------------------------------------
# Name:        stlwriter.py
#
# Author:      Yann Thorimbert
#
# Created:     06.02.2017
#-------------------------------------------------------------------------------
"""
from __future__ import print_function, division


def get_triangles_offset(h, x, y, dh, dx, ox, oy):
    """Returns coords of the 2 triangles with a summit at x,y like this:
            b
         .  .
      .     .
    a . . . c
    and like this:
    d . . . b
    .    .
    . .
    a
    The summit at (x,y) is the bottomleft summit.
    """
    xox = (x+ox)
    yoy = (y+oy)
    bottomleft  =   (xox*dx,    yoy*dx,   h[x,y]*dh)
    topright    =   ((xox+1)*dx,  (yoy+1)*dx, h[x+1,y+1]*dh)
    topleft     =   (xox*dx,    (yoy+1)*dx, h[x,y+1]*dh)
    bottomright =   ((xox+1)*dx,  yoy*dx,   h[x+1,y]*dh)
    return (bottomleft, bottomright, topright), (bottomleft,topright,topleft) #attention ordre


def write_triangle(f,v1,v2,v3):
    s1 = " ".join([format(value,"e") for value in v1])
    s2 = " ".join([format(value,"e") for value in v2])
    s3 = " ".join([format(value,"e") for value in v3])
    f.write("   facet normal 0 0 0\n")
    f.write("       outer loop\n")
    f.write("           vertex " + s1 + "\n")
    f.write("           vertex " + s2 + "\n")
    f.write("           vertex " + s3 + "\n")
    f.write("       endloop\n")
    f.write("   endfacet\n")

def write_stl(hmap, outfile, dh, dx):
    f = open(outfile,"w")
    f.write("solid terrain\n")
    w,h = hmap.shape
    print("Writing STL " + outfile + "...",end="")
    print("width, height = ", w, h)
    for x in range(w-1):
        if x%100 == 0:
            print(round(x*100./w), "%")
        for y in range(h-1):
            for v1,v2,v3 in get_triangles_offset(hmap,x,y,dh,dx,-w//2,-h//2):
                write_triangle(f,v1,v2,v3)
    f.write("endsolid terrain\n")
    f.close()
    print("STL written.")



def write_stl_with_a_roof(hmap, roof_h, outfile, dh, dx):
        f = open(outfile,"w")
        f.write("solid terrain\n")
        w,h = hmap.shape
        # offset = -w//2,-h//2
        offset = 0,0
        print("Writing STL " + outfile + "...",end="")
        print("width, height = ", w, h)
        planar = hmap.copy()
        planar[:,:] = roof_h
        for x in range(w-1):
            if x%200 == 0:
                print(round(x*100./w), "%")
            for y in range(h-1):
                for v1,v2,v3 in get_triangles_offset(hmap,x,y,dh,dx,offset[0],offset[1]):
                    write_triangle(f,v1,v2,v3)
                for v1,v2,v3 in get_triangles_offset(planar,x,y,dh,dx,offset[0],offset[1]):
                    write_triangle(f,v1,v3,v2) #normal pointing opposite direction
        #Write vertical walls
        for x in range(w-1):
            #y = 0 : we link a-c edges (see get_triangles_offset above)
            a,c,b =  get_triangles_offset(hmap,x,0,dh,dx,offset[0],offset[1])[0]
            a2,c2,b2 =  get_triangles_offset(planar,x,0,dh,dx,offset[0],offset[1])[0]
            write_triangle(f,a,a2,c2) #left vertical triangle
            write_triangle(f,a,c2,c) #right vertical triangle
            #y = h-2 : we link d-b edges (see get_triangles_offset above)
            a,b,d =  get_triangles_offset(hmap,x,h-2,dh,dx,offset[0],offset[1])[1]
            a2,b2,d2 =  get_triangles_offset(planar,x,h-2,dh,dx,offset[0],offset[1])[1]
            write_triangle(f,d,b2,d2) #left vertical triangle
            write_triangle(f,d,b,b2) #right vertical triangle
        for y in range(h-1):
            #x = 0 : we link a-d edges (see get_triangles_offset above)
            a,b,d =  get_triangles_offset(hmap,0,y,dh,dx,offset[0],offset[1])[1]
            a2,b2,d2 =  get_triangles_offset(planar,0,y,dh,dx,offset[0],offset[1])[1]
            write_triangle(f,a,d,d2) #top vertical triangle
            write_triangle(f,a,d2,a2) #bottom vertical triangle
            #x = w-2 : we link b-c edges (see get_triangles_offset above)
            a,c,b =  get_triangles_offset(hmap,w-2,y,dh,dx,offset[0],offset[1])[0]
            a2,c2,b2 =  get_triangles_offset(planar,w-2,y,dh,dx,offset[0],offset[1])[0]
            write_triangle(f,b,c,c2) #bottom vertical triangle
            write_triangle(f,b,c2,b2) #top vertical triangle
        f.write("endsolid terrain\n")
        f.close()
        print("STL written.")
