/* This file is part of the xFlows library.
 * Copyright (C) 2009 Jonas Latt
 * E-mail contact: jonas@lbmethod.org
 * The most recent release of xFlows can be downloaded at 
 * <http://www.lbmethod.org/plbows/>
 *
 * The library xFlows is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
e* published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Main author: Orestis Malaspinas
 */

/** \file
 * A collection of dynamics classes (e.g. BGK) with which a Cell object
 * can be instantiated -- header file.
 */
#ifndef ADV_DIFF_TURBULENT_VISCOSITY_DYNAMICS_H
#define ADV_DIFF_TURBULENT_VISCOSITY_DYNAMICS_H

#include "core/globalDefs.h"
#include "core/dynamics.h"
#include "complexDynamics/advectionDiffusionDynamics.h"
namespace plb {
    



/// BGK Advection-Diffusion dynamics
/** This approach contains a slight error in the diffusion
 *  term.
 */
template<typename T, template<typename U> class Descriptor>
class AdvDiff_turbulentViscosity_BGKdynamics : public AdvectionDiffusionDynamics <T,Descriptor> {
public:
    /// Constructor
    AdvDiff_turbulentViscosity_BGKdynamics(T omega_);
    
     /// Return a unique ID for this class.
    virtual int getId() const;
    
    /// Clone the object on its dynamic type.
    virtual AdvDiff_turbulentViscosity_BGKdynamics<T,Descriptor>* clone() const;
    
    /// Collision step
    virtual void collide(Cell<T,Descriptor>& cell,
                         BlockStatistics& statistics_ );
    /// Compute equilibrium distribution function
    virtual T computeEquilibrium(plint iPop, T rhoBar, Array<T,Descriptor<T>::d> const& j,
                                 T jSqr, T thetaBar=T()) const;


    
    

private:
/* *************** Static implementation methods********************** */
    /// Implementation of collision operator
    static T noCorrBGKadvDiff_TurbulenViscosity_Collision (
        Cell<T,Descriptor>& cell, T rhoBar, Array<T,Descriptor<T>::d> const& j, T omega);

};


} // namespace plb

#endif
